@@ -1,21 +1,49 @@
 ### Relation scores.
 
 relation_scores <-
+function(x, method, normalize = FALSE, ...)
+    UseMethod("relation_scores")
+
+relation_scores.relation_ensemble <-
+function(x,
+         method = c("Borda", "Kendall", "differential", "Copeland"),
+         normalize = FALSE, weights = 1, ...)
+{
+    method <- match.arg(method)
+    if(!.is_ensemble_of_endorelations(x))
+        stop("Relation scores are only available for (ensembles of) endorelations.")
+    labs <- LABELS(.domain(x)[[1L]], quote = FALSE)
+    weights <- rep(weights, length.out = length(x))
+    I <- .weighted_sum_of_arrays(lapply(x, .incidence), weights)
+    ret <- switch(method,
+                  Borda =, Kendall = colSums(I),
+                  Copeland =, differential = colSums(I) - rowSums(I)
+                  )
+
+    names(ret) <- labs
+    if(normalize)
+        ret / sum(ret)
+    else
+        ret
+}
+
+
+relation_scores.relation <-
 function(x,
          method = c("ranks", "Barthelemy/Monjardet", "Borda",
-                    "Kendall", "Wei", "differential"),
+                    "Kendall", "Wei", "differential", "Copeland"),
          normalize = FALSE, ...)
 {
     method <- match.arg(method)
     if(!relation_is_endorelation(x))
         stop("Relation scores are only available for endorelations.")
-    labs <- LABELS(relation_domain(x)[[1L]])
+    labs <- LABELS(.domain(x)[[1L]], quote = FALSE)
 
     ## <NOTE>
     ## When adding .relation_scores_FOO(x, ...) method functions, we
     ## might want to pass x as the relation itself rather than its
     ## incidence.
-    I <- relation_incidence(x)
+    I <- .incidence(x)
     ## </NOTE>
 
     ret <- switch(method,
@@ -27,7 +55,7 @@ function(x,
                       (colSums(I * (1 - t(I))) + colSums(I) - 1) / 2
                   },
                   Borda =, Kendall = colSums(I),
-                  differential = colSums(I) - rowSums(I),
+                  Copeland =, differential = colSums(I) - rowSums(I),
                   Wei = {
                       ## <FIXME>
                       ## Cook & Kress use "preference matrices", so to
@@ -35,7 +63,7 @@ function(x,
                       abs(Re(eigen(1 - I)$vectors[, 1L]))
                       ## </FIXME>
                   })
-    
+
     names(ret) <- labs
     if(normalize)
         ret / sum(ret)
@@ -52,4 +80,4 @@ function(x, decreasing = TRUE)
     else
         (n + 1 + colSums(x) - rowSums(x)) / 2
 }
-    
+
